/* $Id: version-info.h 63286 2013-09-12 07:58:57Z vinc17/xvii $ */

/* Written by Vincent Lefevre in 2012 and 2013.
   This file is in the public domain. */

#ifndef VERSION_INFO_STREAM
# define VERSION_INFO_STREAM stdout
#endif

#define VERSION_INFO_STRINGIZE(S) #S
#define VERSION_INFO_MAKE_STR(S) VERSION_INFO_STRINGIZE(S)

#define VERSION_INFO_OUTPUT_ID(S)                       \
  do                                                    \
    {                                                   \
      static char s[] = S;                              \
      char *p = s;                                      \
      while (*++p && (p[0] != ' ' || p[1] != '$'))      \
        putc (p[0], VERSION_INFO_STREAM);               \
      putc ('\n', VERSION_INFO_STREAM);                 \
    }                                                   \
  while (0)

#ifdef SVNID
VERSION_INFO_OUTPUT_ID(SVNID);
#endif

#ifdef SIPE_SVNID
VERSION_INFO_OUTPUT_ID(SIPE_SVNID);
fprintf (VERSION_INFO_STREAM,
         "with SIPE_MINSIZE = %s, SIPE_USE_FMA = %s, SIPE_PREC_MAX = %d, ",
         VERSION_INFO_MAKE_STR (SIPE_MINSIZE),
         VERSION_INFO_MAKE_STR (SIPE_USE_FMA),
         (int) SIPE_PREC_MAX);
#ifdef SIPE_FLOAT
fprintf (VERSION_INFO_STREAM,
         "SIPE_FLOAT = %s\n", VERSION_INFO_MAKE_STR (SIPE_FLOAT));
#elif defined SIPE_ROUND_ZOPT
fprintf (VERSION_INFO_STREAM,
         "SIPE_TRUNC = %s\n", VERSION_INFO_MAKE_STR (SIPE_TRUNC));
fprintf (VERSION_INFO_STREAM,
         " and SIPE_SIZE = %d, SIPE_ROUND_ZOPT = %s\n",
         (int) SIPE_SIZE, VERSION_INFO_MAKE_STR (SIPE_ROUND_ZOPT));
#endif
#endif

#ifdef __GNU_MP_VERSION
fprintf (VERSION_INFO_STREAM,
         "GMP .....  Library: %-12s  Header: %d.%d.%d\n",
         gmp_version, __GNU_MP_VERSION, __GNU_MP_VERSION_MINOR,
         __GNU_MP_VERSION_PATCHLEVEL);
#endif

#ifdef MPFR_VERSION_STRING
fprintf (VERSION_INFO_STREAM,
         "MPFR ....  Library: %-12s  Header: %s (based on %d.%d.%d)\n",
         mpfr_get_version (), MPFR_VERSION_STRING, MPFR_VERSION_MAJOR,
         MPFR_VERSION_MINOR, MPFR_VERSION_PATCHLEVEL);
# if MPFR_VERSION_MAJOR >= 3
fprintf (VERSION_INFO_STREAM, "MPFR features: TLS = %s, decimal = %s",
         mpfr_buildopt_tls_p () ? "yes" : "no",
         mpfr_buildopt_decimal_p () ? "yes" : "no");
#  if MPFR_VERSION_MAJOR > 3 || MPFR_VERSION_MINOR >= 1
fprintf (VERSION_INFO_STREAM, ", GMP internals = %s\nMPFR tuning: %s",
         mpfr_buildopt_gmpinternals_p () ? "yes" : "no",
         mpfr_buildopt_tune_case ());
#  endif
putc ('\n', VERSION_INFO_STREAM);
# endif
# if MPFR_VERSION >= MPFR_VERSION_NUM(2,3,0)
fprintf (VERSION_INFO_STREAM, "MPFR patches: %s\n", mpfr_get_patches ());
# endif
#endif

#ifdef __GLIBC__
# include <gnu/libc-version.h>
fprintf (VERSION_INFO_STREAM, "glibc version: %s\n", gnu_get_libc_version ());
#endif

#ifdef __VERSION__
# if defined(__ICC) || defined(__INTEL_COMPILER)
#  define VERSION_COMPILER " [ICC]"
# elif defined(__GNUC__)
#  define VERSION_COMPILER " [GCC]"
# else
#  define VERSION_COMPILER ""
# endif
fprintf (VERSION_INFO_STREAM, "Compiler version: %s%s\n",
         __VERSION__, VERSION_COMPILER);
#endif

#ifndef VERSION_INFO_NONL
putc ('\n', VERSION_INFO_STREAM);
#endif
